#include "AI.h"

AI::AI(GameRules *gamerules, GameManager *gamemanager, PoolRoom *poolroom)
{
	rules = gamerules;
    manager = gamemanager;
	room = poolroom;
    distance = rules->GetBallSize(0) * 1.9f;
}

AI::~AI()
{

}

void AI::Play(PlayingBall *cue)
{
    float selectedDistance = 0.0f;
    float selectedAngle = 0.0f;
    float differenceAngle = 90.0f;

    int targetBall = -1;
    int targetPocket = -1;

    float cornerPocketAdjust = 0.05f;
    float sidePocketAdjust = 0.05f;

    for(int i = 1; i < rules->GetNumberBalls(); i++)
    {
        if(room->GetPlayingBall(i)->GetObjectState() == CObject::ObjOnTable && (room->GetPlayingBall(i)->GetBallType() == manager->GetCurrentPlayer()->GetTargetType() || manager->GetCurrentPlayer()->GetTargetType() == BALL_NOTARGET))
        {
            for(int j = 0; j < rules->GetNumberPockets(); j++)
            {
                float pocketx = rules->GetPocketX(j);
                float pocketz = rules->GetPocketZ(j) + 1.3f;
                float ballx = room->GetPlayingBall(i)->GetXPos() / SCALE_FACTOR;
                float ballz = room->GetPlayingBall(i)->GetZPos() / SCALE_FACTOR + 1.3f;
                float cuex = room->GetPlayingBall(0)->GetXPos() / SCALE_FACTOR;
                float cuez = room->GetPlayingBall(0)->GetZPos() / SCALE_FACTOR + 1.3f;

                // Don't target the center of the pockets
                float pocketAdjust = rules->GetPocketRadius(j);
                switch(j)
                {
                    case 0:
                        pocketx += pocketAdjust - cornerPocketAdjust;
                        pocketz += pocketAdjust - cornerPocketAdjust;
                        break;
                    case 1:
                        pocketx += pocketAdjust - cornerPocketAdjust;
                        pocketz -= pocketAdjust - cornerPocketAdjust;
                        break;
                    case 2:
                        pocketx -= pocketAdjust - cornerPocketAdjust;
                        pocketz -= pocketAdjust - cornerPocketAdjust;
                        break;
                    case 3:
                        pocketx -= pocketAdjust - cornerPocketAdjust;
                        pocketz += pocketAdjust - cornerPocketAdjust;
                        break;
                    case 4:
                        pocketx += pocketAdjust + sidePocketAdjust;
                        break;
                    case 5:
                        pocketx -= pocketAdjust + sidePocketAdjust;
                        break;
                    default:
                        break;
                }

	            float targetx, targetz;
	            float ballToPocketAngle = CalculateAngle( ballx, ballz, pocketx, pocketz );
                float cueToBallAngle = CalculateAngle( cuex, cuez, ballx, ballz );

                // Disregard as a possible shot if not ball between cue and pocket
                if ( fabs( DegToRad( To360Angle(ballToPocketAngle, ballx, ballz, pocketx, pocketz) - To360Angle(cueToBallAngle, cuex, cuez, ballx, ballz) ) ) <= 60.0 )
                {
                    CalculateTargetPosition( &targetx, &targetz, pocketx, pocketz, ballx, ballz, ballToPocketAngle);

	                float cueToTargetAngle = CalculateAngle( cuex, cuez, targetx, targetz );

                    ballToPocketAngle = To360Angle(ballToPocketAngle, ballx, ballz, pocketx, pocketz);
                    cueToTargetAngle = To360Angle(cueToTargetAngle, cuex, cuez, targetx, targetz);

                    // If it's a smaller angle than any previous shot
                    if( fabs(cueToTargetAngle - ballToPocketAngle) < differenceAngle )
                    {
                        bool obstructed = false;
                        float checkx, checkz;
                        float currentDistance, cueToTargetDistance, ballToPocketDistance;

                        currentDistance = rules->GetBallSize(0);
                        cueToTargetDistance = CalculateDistance(cuex, cuez, targetx, targetz);

                        // Checking between cue and target
                        while( !obstructed && currentDistance <= cueToTargetDistance )
                        {
                            CalculatePosition( checkx, checkz, cuex, cuez, cueToTargetAngle, currentDistance);
                            for( int ball = 1; ball < rules->GetNumberBalls(); ball++)
                            {
                                if(ball != i)
                                {
                                    float dist = CalculateDistance( checkx, checkz, room->GetPlayingBall(ball)->GetXPos() / SCALE_FACTOR, room->GetPlayingBall(ball)->GetZPos() / SCALE_FACTOR + 1.3f);
                                    if( dist < rules->GetBallSize(0) * 2 )
                                        obstructed = true;
                                }
                            }
                            currentDistance += rules->GetBallSize(0);
                        }

                        currentDistance = rules->GetBallSize(0);
                        ballToPocketDistance = CalculateDistance(ballx, ballz, pocketx, pocketz);

                        // Checking between ball and pocket
                        while( !obstructed && currentDistance <= ballToPocketDistance )
                        {
                            CalculatePosition( checkx, checkz, ballx, ballz, ballToPocketAngle, currentDistance);
                            for( int ball = 1; ball < rules->GetNumberBalls(); ball++)
                            {
                                if(ball != i)
                                {
                                    float dist = CalculateDistance( checkx, checkz, room->GetPlayingBall(ball)->GetXPos() / SCALE_FACTOR, room->GetPlayingBall(ball)->GetZPos() / SCALE_FACTOR + 1.3f);
                                    if( dist < rules->GetBallSize(0) * 2 )
                                        obstructed = true;
                                }
                            }
                            currentDistance += rules->GetBallSize(0);
                        }

                        // If there is nothing in the way
                        if( !obstructed )
                        {
                            differenceAngle = fabs(cueToTargetAngle - ballToPocketAngle);
                            selectedAngle = cueToTargetAngle;

                            // Visual targetting
                            //room->target->SetModelPosition(targetx * SCALE_FACTOR, rules->GetBallSize(0) * SCALE_FACTOR, (targetz - 1.3) * SCALE_FACTOR);
                            //room->targetPocket->SetModelPosition(pocketx * SCALE_FACTOR, rules->GetBallSize(0) * SCALE_FACTOR, (pocketz - 1.3) * SCALE_FACTOR);
                            //room->targetBall->SetModelPosition(ballx * SCALE_FACTOR, rules->GetBallSize(0) * SCALE_FACTOR, (ballz - 1.3) * SCALE_FACTOR);
                            //room->startingPoint->SetModelPosition(cuex * SCALE_FACTOR, rules->GetBallSize(0) * SCALE_FACTOR, (cuez - 1.3) * SCALE_FACTOR);
                        }
                    }
                }
            }
        }
    }

	float force = 80.0f;
    cue->AddForce(force * (float)(cos(selectedAngle) * 1), 0, force * (float)(sin(selectedAngle) * -1));
}

float AI::CalculateDistance(float x1, float z1, float x2, float z2)
{
	return sqrt( ( (z2 - z1) * (z2 - z1) ) + ( (x2 - x1) * (x2 - x1) ) );
}

float AI::CalculateAngle(float x1, float z1, float x2, float z2)
{
	return atan( fabs(z2 - z1) / fabs(x2 - x1) );
}

void AI::CalculateTargetPosition(float *targetX, float *targetZ, float pocketX, float pocketZ, float ballX, float ballZ, float angle)
{
	if ( ballX < pocketX )
		*targetX = ballX - ( cos(fabs(angle)) * distance );
	else
		*targetX = ballX + ( cos(fabs(angle)) * distance );
	if ( ballZ < pocketZ )
		*targetZ = ballZ - ( sin(fabs(angle)) * distance );
	else
		*targetZ = ballZ + ( sin(fabs(angle)) * distance );
}

void AI::CalculatePosition(float &x, float &z, float fromx, float fromz, float angle, float dist)
{
    if ( RadToDeg(angle) >= 90 && RadToDeg(angle) <= 270 )
		x = fromx - ( cos(fabs(angle)) * dist );
	else
		x = fromx + ( cos(fabs(angle)) * dist );
	if ( RadToDeg(angle) >= 180 && RadToDeg(angle) <= 360 )
		z = fromz - ( sin(fabs(angle)) * dist );
	else
		z = fromz + ( sin(fabs(angle)) * dist );
}

float AI::To360Angle(float angle, float x1, float z1, float x2, float z2)
{
    if(x1 < x2 && z1 > z2)
		return angle;
	if(x1 > x2 && z1 > z2)
		return (PI - angle);
	if(x1 > x2 && z1 < z2)
		return (PI + angle);
	if(x1 < x2 && z1 < z2)
		return (TWOPI - angle);
}

float AI::DegToRad(float angle)
{
    return angle * PI / 180;
}

float AI::RadToDeg(float angle)
{
    return angle * 180 / PI;
}
