//CObject.cpp

#include "CObject.h"

CObject::CObject()
{
	InitialiseVariables();
}

CObject::CObject(char *modelName, ModelController *modelController, char* objName)
{
    InitialiseVariables();
	if( objName != NULL )
		name = objName;

    if( modelName != NULL )
        model = modelController->LoadModel(modelName);
}

CObject::~CObject()
{ 
	if (HasChild())
		delete childNode;
	if (HasParent() && !IsLastChild())
        delete nextNode;
}

void CObject::InitialiseVariables()
{
	LoadIdentity(&rotation);
    posX     = 0.0f;
	posY     = 0.0f;
	posZ     = 0.0f;
    model    = NULL;
    bodyID   = NULL;
    name     = "CObject";
    objState = ObjOnTable;
}

void CObject::Draw(Mat44 worldtoscreen, lightinfo lights) 
{
	OnDraw(worldtoscreen, lights);

	// draw children	
	if (HasChild())
		((CObject*)childNode)->Draw(worldtoscreen,lights);

	// draw siblings
	if (HasParent() && !IsLastChild())
		((CObject*)nextNode)->Draw(worldtoscreen,lights);
}

void CObject::OnDraw(Mat44 worldtoscreen, lightinfo lights)
{
   UpdateModelPosition();
   if( model != NULL )
   {
        LoadIdentity(&rotation);
        if(bodyID != NULL)
        {
            const dReal* odeQ = dBodyGetQuaternion(bodyID);
            const float a = odeQ[0];
            Vec3 v = Vec3(odeQ[1], odeQ[2], odeQ[3]);
            Quat qu(v, a);
            LoadRotation(&rotation, qu);
        }
        //Translate model to correct spot
        SetColumn(&rotation, 3, Vec4(posX, posY, posZ, 1.0));
        model->DrawModel(rotation, worldtoscreen, lights);
	}
}

void CObject::SetModelPosition(float x, float y, float z)
{
    posX = x;
    posY = y;
    posZ = z;
}

float CObject::GetXPos()
{
    return posX;
}

float CObject::GetYPos()
{
    return posY;
}

float CObject::GetZPos()
{
    return posZ;
}

void CObject::AddForce(dReal x, dReal y, dReal z)
{
    if( bodyID != NULL )
        dBodyAddForce(bodyID, x, y, z);
}

void CObject::StopMovement()
{
    if( bodyID != NULL )
    {
	    dBodySetLinearVel(bodyID, 0.0, 0.0, 0.0);
	    dBodySetAngularVel(bodyID, 0.0, 0.0, 0.0);
        const dReal* fptr = dBodyGetForce(bodyID);
        dBodyAddForce(bodyID, -fptr[0], -fptr[1], -fptr[2]);
    }
}

dBodyID CObject::GetBodyID()
{
    return bodyID;
}

void CObject::SetPhysicalPosition(float x, float y, float z)
{
    if( bodyID != NULL )
        dBodySetPosition(bodyID, x, y, z);
}

void CObject::UpdateModelPosition()
{
    if( bodyID != NULL )
    {
        const dReal *currentPos = dBodyGetPosition(bodyID);
	    posX = currentPos[0] * SCALE_FACTOR;
	    posY = currentPos[1] * SCALE_FACTOR;
	    posZ = currentPos[2] * SCALE_FACTOR;
    }
}

char* CObject::GetName()
{
    return name;
}

void CObject::DrawObject(Mat44 worldtoscreen, lightinfo lights)
{
    OnDraw(worldtoscreen, lights);
}

void CObject::SetObjectState(ObjectState newState)
{
   objState = newState;    
}

int CObject::GetObjectState()
{
    return (int)objState;
}
