//CObject - the base class for all GameWorld objects

#ifndef __OBJECT_H
#define __OBJECT_H

#include "CNode.h"
#include <stdio.h>
#include <string.h>
#include "Model.h"
#include "ModelController.h"
#include "GeomController.h"
#include "Constants.h"
#include <ode/ode.h>
#include <geommath/geommath.h>

class CObject : public CNode
{
public:
    enum ObjectState
    {
        ObjOnTable  = 0,
        ObjOffTable = 1,
        ObjInPocket = 2
    };

	CObject();
	CObject(char *modelName, ModelController *modelController, char* objName);
	virtual ~CObject();
   
	void    SetModelPosition(float x, float y, float z);
    void    SetPhysicalPosition(float x, float y, float z);
	void    Draw(Mat44 worldtoscreen, lightinfo lights);
    float   GetXPos(); //MODEL position, NOT physical (physical position = model position / SCALE_FACTOR)
    float   GetYPos(); //MODEL position, NOT physical
    float   GetZPos(); //MODEL position, NOT physical
    void    AddForce(dReal x, dReal y, dReal z);
    void    StopMovement();
    dBodyID GetBodyID();
    char*   GetName();
    void    DrawObject(Mat44 worldtoscreen, lightinfo lights);
    int     GetObjectState();
    void    SetObjectState(ObjectState newState);

protected:
	virtual void OnDraw(Mat44 worldtoscreen, lightinfo lights);
    void UpdateModelPosition();
    void InitialiseVariables();

	float       posX, posY, posZ;
	Mat44       rotation;
	Model*      model;
    dMass       mass;
    dBodyID     bodyID;
    char*       name;
    ObjectState objState;
};

#endif
