//Camera.cpp
//
//Implementation of Camera class

#include "Camera.h"

Camera::Camera()
{
   cameraToScreen = Render_MakeCameraToScreen();
   restrictCamera = false;
   maxAngle       = PI * 1.5;
   minAngle       = PI * 0.5;
   InitialiseVariables();
}

Camera::~Camera()
{}

void Camera::InitialiseVariables()
{
    rotX           = 0.0;
    rotY           = 0.0;
    rotZ           = 0.0;
    offsetX        = 0.0;
    offsetY        = -5000.0;
    offsetZ        = -10000.0;
    minZoom        = -8000.0;
    maxZoom        = -14000.0;
    cueZOffset     = 8000.0;
}

Mat44 Camera::MakeWorldToScreenMatrix(float centreX, float centreY, float centreZ)
{
    //Matrix centring view on white ball   
    LoadIdentity(&tempWC);
    SetColumn(&tempWC, 3, Vec4(-centreX, -centreY, -centreZ, 1.0f));
         
    //Matrix rotating around white ball   
    LoadIdentity(&tempWC2);
    LoadRotation(&tempWC2, rotX, rotY, rotZ);

    //Matrix moving away from white ball to final camera position
    LoadIdentity(&tempWC3);
    SetColumn(&tempWC3, 3, Vec4(offsetX, offsetY, offsetZ, 1.0f));
       
    LoadIdentity(&tempWC4);
    LoadRotation(&tempWC4, PI/8, 0.0, 0.0);

    return (cameraToScreen * tempWC4 * tempWC3 * tempWC2 * tempWC);
}

Mat44 Camera::MakeCueMatrix(float centreX, float centreY, float centreZ)
{
    //Matrix centring view on white ball   
    LoadIdentity(&tempWC);
    SetColumn(&tempWC, 3, Vec4(-centreX, -centreY, -centreZ, 1.0f));
         
    //Matrix rotating around white ball   
    LoadIdentity(&tempWC2);
    LoadRotation(&tempWC2, rotX, rotY, rotZ);

    //Matrix moving away from white ball to final camera position
    LoadIdentity(&tempWC3);
    SetColumn(&tempWC3, 3, Vec4(offsetX, offsetY, offsetZ, 1.0f));

    Mat44 tempMat, tempMat2;
    LoadIdentity(&tempMat);
    LoadRotation(&tempMat, HALFPI, 0.0, 0.0);//1.698f, 0.0, 0.0);

    LoadIdentity(&tempMat2);
    SetColumn(&tempMat2, 3, Vec4(0.0, -offsetY, 0.0, 1.0));// - 198.0f, offsetZ, 1.0));
    
    return (cameraToScreen * tempMat2 * tempMat * tempWC3 * tempWC2 * tempWC);
}

void Camera::RotateCamera(float dx, float dy, float dz)
{
   rotX += dx;
   if( rotX < 0.0 )
      rotX += TWOPI;
   if( rotX > TWOPI )
      rotX -= TWOPI;

   rotY += dy;
   if( rotY < 0.0 )
      rotY += TWOPI;
   if( rotY > TWOPI )
      rotY -= TWOPI;
   if(restrictCamera)
   {
       if((rotY > minAngle) && (rotY <= PI))
           rotY = minAngle;
       if((rotY < maxAngle) && (rotY >= PI))
           rotY = maxAngle;
   }

   rotZ += dz;
   if( rotZ < 0.0 )
      rotZ += TWOPI;
   if( rotZ > TWOPI )
      rotZ -= TWOPI;
}

void Camera::GetCameraVector(float resultVec[3])
{
   resultVec[0] = sinf(rotY);
   resultVec[1] = 0.0;
   resultVec[2] = -cosf(rotY);
}

float Camera::GetXRotation()
{
   return rotX;
}

float Camera::GetYRotation()
{
   return rotY;
}

float Camera::GetZRotation()
{
   return rotZ;
}

float Camera::GetXOffset()
{
   return offsetX;
}

float Camera::GetYOffset()
{
   return offsetY;
}

float Camera::GetZOffset()
{
   return offsetZ;
}

void Camera::Zoom(float amount)
{
   offsetZ += amount;
   if( offsetZ > minZoom )
      offsetZ = minZoom;
   if( offsetZ < maxZoom )
      offsetZ = maxZoom;
}

void Camera::ResetCamera()
{
   InitialiseVariables();
}

void Camera::RestrictCamera(bool doRestrict)
{
    restrictCamera = doRestrict;
}

void Camera::SetMaxAngle(float newMaxAngle)
{
    maxAngle = newMaxAngle;
}

void Camera::SetMinAngle(float newMinAngle)
{
    minAngle = newMinAngle;
}
