#include "Debugger.h"

Debugger::Debugger()
{
	printf("Starting new instance of debugger:..");
	s = new SocketController();
	SockNo = s->servSock;
	keepgoing = true;
	ticker = 0;
	interval=100;
	z=0;
	s->SendMessage("!STARTED~DONE~");
	c = NULL;
	p = NULL;
	g = NULL;
}

Debugger::~Debugger()
{
	s->SendMessage("DIE~DONE~");
	s->CloseConnection();
	printf("\nKilling Debugger[");
	delete s;
	printf(",~Debugger]\n");
}

void Debugger::SetCamera(Camera *cam)
{
   c = cam;
}

void Debugger::SetPoolRoom(PoolRoom *pr)
{
	p = pr;
}

void Debugger::SetGameRules(GameRules *gr)
{
	g = gr;
}

void Debugger::Post(char msg[BUFLEN])
{
	char message[BUFLEN];
	sprintf(message,"msg|%s~DONE~",msg);
	s->SendMessage(message);
}

void Debugger::PostData(char msg[BUFLEN])
{
	char message[BUFLEN];
	sprintf(message,"%s~",msg);
	s->SendMessage(message);
}

void Debugger::UpdateAll()
{
   //do the cobject tree.
   TraverseTree(p);
   //do the camera
   char msg[BUFLEN];
   msg[0]='\0';
   sprintf(msg, "list|Camera|Camera|rotX = %f,rotY = %f,rotZ = %f,offsetX = %f,offsetY = %f,offsetZ = %f~DONE~\0", c->GetXRotation(), c->GetYRotation(), c->GetZRotation(), c->GetXOffset(), c->GetYOffset(), c->GetZOffset());
   s->SendMessage(msg);
}

void Debugger::TraverseTree(CObject* obj)
{
	char msg[BUFLEN];
	msg[0]='\0';
	sprintf(msg, "list|PoolRoom|%s|posX = %f,posY = %f,posZ = %f~DONE~", obj->GetName(), obj->GetXPos(), obj->GetYPos(), obj->GetZPos());
	s->SendMessage(msg);
	// draw children	
	if (obj->HasChild()) TraverseTree((CObject*)obj->childNode);

	// draw siblings
	if (obj->HasParent() && !(obj->IsLastChild())) TraverseTree((CObject*)obj->nextNode);
}

bool Debugger::OhPlease()
{
	char msg[BUFLEN];
	msg[0]='\0';
	if (s->servSock == -1) 
	{
		delete s;
		s = new SocketController();
		SockNo = s->servSock;
	}
	int ret = s->CheckMessages(msg);
	if (ret == 3)
	{
		if (!keepgoing)
			s->SendMessage("!STOPPED~");
		else
			s->SendMessage("!STARTED~");
	}
	else if (ret > 0)
	{
		if (strlen(msg)>0) printf("%s\n",msg);
		if (strncmp(msg,"STOP",4)==0)
		{
			keepgoing = !keepgoing;
			if (!keepgoing)
				s->SendMessage("!STOPPED~");
			else
				s->SendMessage("!STARTED~");
		}
		else if (strncmp(msg,"CLOSE",5)==0)
		{
			s->SendMessage("DIE");
			s->CloseConnection();
		}
		else if (strncmp(msg,"INT",3)==0)
		{
			char *a = msg, *b = strstr(a,"T");
			b++;
			int newInterval = atoi(b);
			printf("convert:%d\n",newInterval);
			interval = newInterval * 10;
		}
	}
	
	ticker++;
	if (ticker > interval)
	{
		ticker=0;
		UpdateAll();
	}
	return keepgoing;
}