//GameManager.cpp

#include "GameManager.h"

GameManager::GameManager(GameRules *rulesSet, PoolRoom *room, Camera *cam)
{
    players[0]    = new Player("Player 1", 1);
    players[1]    = new Player("Player 2", 2);
    currentPlayer = players[0];
    otherPlayer   = players[1];
    winner        = NULL;
    loser         = NULL;
    gameRules     = rulesSet;
    poolRoom      = room;
    camera        = cam;
    firstBallHit  = false;
    foulFlag      = false;
    continueFlag  = false;
    endGame       = false;
}

GameManager::~GameManager()
{
    delete players[0];
    delete players[1];
}

void GameManager::ProcessCollision(PlayingBall *pBall1, PlayingBall *pBall2)
{
    if(!firstBallHit)
    {
        if(pBall1->GetBallType() == BALL_WHITE)
        { 
            firstBallHit = true;
            if((currentPlayer->GetTargetType() != BALL_NOTARGET) && (currentPlayer->GetTargetType() != pBall2->GetBallType()))
                foulFlag = true;
        }   
            
        if(pBall2->GetBallType() == BALL_WHITE)
        {
            firstBallHit = true;
            if((currentPlayer->GetTargetType() != BALL_NOTARGET) && (currentPlayer->GetTargetType() != pBall1->GetBallType()))
                foulFlag = true;
        }
    }
}

void GameManager::ProcessCollision(PlayingBall *pBall, CollisionType cType)
{
    if(cType == BallPocket)
    {
        pBall->RemoveFromTable();
        pBall->SetObjectState(CObject::ObjInPocket);

        switch(pBall->GetBallType())
        {
            case BALL_WHITE:
            {
                foulFlag = true;
                camera->RestrictCamera(true);
                camera->ResetCamera();
                break;
            }

            case BALL_BLACK:
            {
                if(currentPlayer->GetTargetType() == BALL_BLACK)
                {
                    if(foulFlag)
                    {
                        winner  = otherPlayer;
                        loser   = currentPlayer;
                    }
                    else
                    {
                        winner  = currentPlayer;
                        loser   = otherPlayer;
                    }
                    endGame = true;
                }
                else
                {
                    winner  = otherPlayer;
                    loser   = currentPlayer;
                    endGame = true;
                }

                break;
            }

            default:
            {
                if(currentPlayer->GetTargetType() == BALL_NOTARGET)
                {
                    continueFlag = true;
                    currentPlayer->SetTargetType(pBall->GetBallType());
                    if(gameRules->GetOppositeType(pBall->GetBallType()) != BALL_NOOPPOSITE)
                        otherPlayer->SetTargetType(gameRules->GetOppositeType(pBall->GetBallType()));
                }
                else
                {   
                    if(pBall->GetBallType() == currentPlayer->GetTargetType())
                        continueFlag = true;
                    else
                        foulFlag = true;
                }
            }
        }

        if(pBall->GetRespawnFromPocketStatus())
            pBall->SetRespawnStatus(true);
    }

    if(cType == BallRoom)
    {
        pBall->RemoveFromTable();
        pBall->SetObjectState(CObject::ObjOffTable);
        pBall->SetRespawnStatus(true);
        foulFlag = true;
    }
}

void GameManager::EndTurn()
{
    poolRoom->RespawnBalls();
    CheckFoulContinueFlags();
    
    for(int i = 0; i < 2; ++i)
        if(AllOfTypeSunk(players[i]->GetTargetType()))
            players[i]->SetTargetType(BALL_BLACK);

    firstBallHit = false;
    foulFlag     = false;
    continueFlag = false;
}

bool GameManager::AllOfTypeSunk(int ballType)
{
    if(ballType == BALL_NOTARGET)
        return false;

    bool allSunk = true;
    PlayingBall *pBall;

    for(int i = 0; i < gameRules->GetNumberBalls(); ++i)
    {
        pBall = poolRoom->GetPlayingBall(i);
        if((pBall->GetBallType() == ballType) && (pBall->GetObjectState() != CObject::ObjInPocket))
            allSunk = false;
    }

    return allSunk;
}

void GameManager::CheckFoulContinueFlags()
{
    if(foulFlag)
        SwapPlayer(2);
    else
    {
        if(!firstBallHit)
            SwapPlayer(2); //No balls were hit by the white ball during the turn
        else
        {
            if(continueFlag && currentPlayer->GetNumberOfTurnsLeft() == 1)
                currentPlayer->ChangeNumberOfTurns(1);

            currentPlayer->ChangeNumberOfTurns(-1);
            if(currentPlayer->GetNumberOfTurnsLeft() <= 0)
                SwapPlayer(1);
        }
    }
}

void GameManager::SwapPlayer(int numTurns)
{
    if(players[0] == currentPlayer)
    {
        currentPlayer = players[1];
        otherPlayer   = players[0];
    }
    else
    {
        currentPlayer = players[0];
        otherPlayer   = players[1];
    }

    currentPlayer->SetNumberOfTurns(numTurns);
}

bool GameManager::IsGameOver()
{
    return endGame;
}

Player* GameManager::GetPlayer(int playerNum)
{
    if(playerNum < 0 || playerNum > 1)
        return players[0];
    else
        return players[playerNum];
}

Player* GameManager::GetCurrentPlayer()
{
    return currentPlayer;
}

char* GameManager::GetFirstBallHitStatus()
{
    if(firstBallHit)
        return "First ball hit";
    else
        return "First ball not hit";
}

char* GameManager::GetFoulFlagStatus()
{
    if(foulFlag)
        return "Foul occured";
    else
        return "No fouls";
}

char* GameManager::GetContinueFlagStatus()
{
    if(continueFlag)
        return "Continue occured";
    else
        return "No continues";
}

Player* GameManager::GetWinner()
{
    return winner;
}

Player* GameManager::GetLoser()
{
    return loser;
}

void GameManager::StartNewGame()
{
    players[0]->ResetPlayer();
    players[1]->ResetPlayer();
    currentPlayer = players[0];
    otherPlayer   = players[1];
    winner        = NULL;
    loser         = NULL;
    firstBallHit  = false;
    foulFlag      = false;
    continueFlag  = false;
    endGame       = false;   
    poolRoom->ResetAllBalls();
    camera->ResetCamera();
}