//GameManager.h
//
//Manages the game of billiards

#ifndef GAME_MANAGER_H
#define GAME_MANAGER_H

#include "GameRules.h"
#include "PoolRoom.h"
#include "PlayingBall.h"
#include "Player.h"
#include "Camera.h"
#include "Constants.h" //Contains the CollisionType enum

class GameManager
{
public:
    GameManager(GameRules *rulesSet, PoolRoom *room, Camera *cam);
    ~GameManager();

    void    ProcessCollision(PlayingBall *pBall1, PlayingBall *pBall2);
    void    ProcessCollision(PlayingBall *pBall, CollisionType cType);
    void    EndTurn();
    bool    IsGameOver();
    Player* GetPlayer(int playerNum);
    Player* GetCurrentPlayer();
    Player* GetWinner();
    Player* GetLoser();
    void    StartNewGame();

    char* GetFirstBallHitStatus();
    char* GetFoulFlagStatus();
    char* GetContinueFlagStatus();

private:
    bool AllOfTypeSunk(int ballType);
    void CheckFoulContinueFlags();
    void SwapPlayer(int numTurns);
    
    Player    *players[2], *currentPlayer, *otherPlayer, *winner, *loser;
    GameRules *gameRules;
    PoolRoom  *poolRoom;
    Camera    *camera;

    bool firstBallHit, foulFlag, continueFlag, endGame;
};

#endif
