#include "HUD.h"

HUD::HUD(TextureManager *t, GameManager *ptrGameManager, int sps2d)
{
    desc = sps2d;
	menuItem = MENU_START1PLAYER;
	texman = t;

    texman->LoadFile("logo.bmp",true);
    texman->LoadFile("PINPLogo.bmp",true);
    texman->LoadFile("menu.bmp",true);
	texman->LoadFile("ball.bmp",true);
    texman->LoadFile("8Ball.bmp",true);
    texman->LoadFile("LowBall.bmp",true);
    texman->LoadFile("HighBall.bmp",true);
    texman->LoadFile("CueBall.bmp",true);
    texman->LoadFile("UnknownTarget.bmp",true);
    texman->LoadFile("one.bmp",true);
    texman->LoadFile("two.bmp",true);
    texman->LoadFile("GameOverOne.bmp",true);
    texman->LoadFile("GameOverTwo.bmp",true);
    texman->LoadFile("powermeter.bmp",true);
	texman->LoadFile("newscoreboard.bmp",true);

	introString = "Press START to continue!";

	menuItemY[0] = 40;
	menuItemY[1] = 72;
	menuItemY[2] = 119;
	menuItemY[3] = 160;
    playerY[0] = 16;
    playerY[1] = 48;
    cPlayer = playerY[0];

	gameManager = ptrGameManager;

    targets[0] = "High balls";
    targets[1] = "Low balls";
    targets[2] = "Black ball";
    targets[3] = "White ball";
    targets[4] = "No target";
    shootingForce = 0.0;
    maxForce      = -1.0;

    pMemory=sps2Allocate(4096, SPS2_MAP_BLOCK_4K | SPS2_MAP_UNCACHED,desc);

	introLogoX = 0;
	introLogoY = (480/2);
	introTextX = 640-sps2UPrintfStringWidth(&fontHelvetica18,introString);
	introTextY = 380;
	introMoveDelta = 26;

    credits = new Credits("Credits.txt");
}

void HUD::setShootingForce(float force)
{
    shootingForce = force;
}

void HUD::setMaxForce(float _maxForce)
{
    maxForce = _maxForce;
}

HUD::~HUD()
{
    delete credits;
    sps2Free(pMemory);
}	

void HUD::drawHUD(int gamestate, int sockNo)
{
	if (gamestate == GAMESTATE_INTRO)
	{
		int stringXMin = 320-(sps2UPrintfStringWidth(&fontHelvetica18,introString)/2);
		introMoveDelta -= 1;
		if (introMoveDelta < 1) introMoveDelta = 1;
		introLogoX += introMoveDelta;
		if (introLogoX > 320) introLogoX = 320;
		introTextX -= introMoveDelta;
		if (introTextX < stringXMin) introTextX = stringXMin;
		texman->DrawBitmap("PINPLogo.bmp",introLogoX-(256/2), introLogoY-(256/2), 16777215);
		sps2UPrintf(&fontHelvetica18, "\377FFFFFF\376(%i,%i)%s",introTextX,introTextY,introString);
	}
	else if (gamestate != GAMESTATE_OUTRO)
	{
		texman->DrawBitmap("logo.bmp",512, 0, 16777215);
	}
	if(gamestate != GAMESTATE_SHOWRESULTS && gamestate != GAMESTATE_INTRO && gamestate != GAMESTATE_OUTRO)
    {
        texman->DrawBitmap("newscoreboard.bmp",0,0,15080000);

        cPlayer = playerY[gameManager->GetCurrentPlayer()->GetPlayerNumber() - 1];
        texman->DrawBitmap("CueBall.bmp", 14, cPlayer, 15090000);
        
        int target = gameManager->GetCurrentPlayer()->GetTargetType();
        switch(gameManager->GetCurrentPlayer()->GetNumberOfTurnsLeft())
        {
            case 1:
                texman->DrawBitmap("one.bmp", 200, cPlayer, 15090000);
                break;
            case 2:
                texman->DrawBitmap("two.bmp", 200, cPlayer, 15090000);
                break;
        }

        DrawTargetGroup(gameManager->GetPlayer(0)->GetTargetType(), 150, playerY[0]);
        DrawTargetGroup(gameManager->GetPlayer(1)->GetTargetType(), 150, playerY[1]);
        DrawPowerMeter(320, 10, shootingForce/maxForce*100);
    }

    if(gamestate == GAMESTATE_MENU)
        drawMenu();

    if(gamestate == GAMESTATE_SHOWRESULTS)
    {
        switch(gameManager->GetWinner()->GetPlayerNumber())
        {
            case 1:
                texman->DrawBitmap("GameOverOne.bmp", 0, 0, 15090000);
                break;
            case 2:
                texman->DrawBitmap("GameOverTwo.bmp", 0, 0, 15090000);
                break;
        }
    }

    if(gamestate == GAMESTATE_OUTRO)
    {
  		texman->DrawBitmap("logo.bmp", 256, 10, 16777215);
        credits->ShowCredits();
    }

	if (sockNo == -1)
		sps2UPrintf(&fontHelvetica18, "\377FFFFFF\376(620,400)*", sockNo);
    
    sps2UPrintfRender();
	DPUT_GS_BGCOLOR(0);
}

void HUD::drawMenu()
{
	int menux = 192, menuy = 112;
	texman->DrawBitmap("menu.bmp",menux, menuy,16000000);
	texman->DrawBitmap("ball.bmp",menux + 18, menuy + menuItemY[menuItem],16010000);
}

int HUD::getCurrentSelection()
{
	return menuItem;
}

void HUD::changeMenuItem(int dir)
{
	menuItem += dir;
	if (menuItem < 0) menuItem = MAX_MENU_ITEM - 1;
	if (menuItem > (MAX_MENU_ITEM - 1)) menuItem = 0;
}

void HUD::DrawTargetGroup(int targetGroup, int x, int y)
{
    switch(targetGroup)
    {
        case BALL_NOTARGET:
            texman->DrawBitmap("UnknownTarget.bmp", x, y, 15090000);
            break;
        case BALL_SMALL:
            texman->DrawBitmap("LowBall.bmp", x, y, 15090000);
            break;
        case BALL_BIG:
            texman->DrawBitmap("HighBall.bmp", x, y, 15090000);
            break;
        case BALL_BLACK:
            texman->DrawBitmap("8Ball.bmp", x, y, 15090000);
            break;
    }
}

void HUD::DrawPowerMeter(int x, int y, float shotStrength)
{
    texman->DrawBitmap("powermeter.bmp", x, y, 16000000);
    DrawQuad(x + 8, y + 10, 1.13 * shotStrength, 18,16100000);
}

void HUD::DrawQuad(int x, int y, int w, int h, int z)
{
  	int iQWC;
    Dn_CHCR_t chcrValue;      // This is used to control the DMA transfer
    chcrValue.i32=0;                  // Blank everything out
    chcrValue.s.MOD=CHCR_MOD_NORMAL;  // We'll do a "normal" dma transfer
    chcrValue.s.STR=1;                // STR=1 = start transfer
  
    iQWC=loadTriangleData(pMemory->pvStart, x,y,w,h, z);
    *EE_D2_QWC=iQWC;
    *EE_D2_MADR=sps2GetPhysicalAddress(pMemory->pvStart, pMemory);
    *EE_D2_CHCR=chcrValue.i32;
    sps2WaitForDMA(2, desc);
}

int HUD::loadTriangleData(void *pvBase, int x, int y, int w, int h, int z) {
	x += -320;
	y += -240;
    vertex_t aVertices[]={
		{x, y}, {x, y + h}, {x + w, y}, {x + w, y + h}
    };
    int iVertexCount=sizeof(aVertices)/sizeof(vertex_t);
    sps2GIFTag_t *pGIFTag=(sps2GIFTag_t *) pvBase;
    sps2GIFPackedRegister_t *pRegister=(sps2GIFPackedRegister_t *) ((int) pvBase+sizeof(sps2GIFTag_t));
    int iVertex;
    // Prepare the GIF tag  
    pGIFTag->i128=0;       // Blank it out
    pGIFTag->s.NLOOP=0;    // 0 entries.  We'll increment as we add entries
    pGIFTag->s.PRE=1;      // We are providing a valid PRIM value
    pGIFTag->s.EOP=1;      // End of packet -- draw me please!
    pGIFTag->s.PRIM=12;    // 3 = triangle | 8 = gourard shading
    pGIFTag->s.FLG=GIF_FLG_PACKED; // We're using the packed format for data
    pGIFTag->s.NREG=0;     // Start off with zero registers, increment as we
                            // populate the descriptions
      
    // Add a register.  It will be an RGBAQ register
    SPS2_SET_GIF_REG(*pGIFTag, pGIFTag->s.NREG++, GIF_REG_RGBAQ);
    // Add another register.  It will by an XYZ2 register
    SPS2_SET_GIF_REG(*pGIFTag, pGIFTag->s.NREG++, GIF_REG_XYZ2);
      
    // Loop through all vertices
    for (iVertex=0;iVertex<iVertexCount;iVertex++) {
        pGIFTag->s.NLOOP++;  // Increment the entry count in the tag
        
        pRegister->i128=0;   // Blank out this value
        pRegister->RGBAQ.s.A=0xff; // Alpha = 127
        // Pick R, G, and B values just based on the vertex number and frame
        pRegister->RGBAQ.s.R=(0xff);
        pRegister->RGBAQ.s.G=(0x0);
        pRegister->RGBAQ.s.B=(0x0);
        
        // Increment register data pointer
        pRegister++;
        
        pRegister->i128=0;  // Blank out this value
		pRegister->XYZ2.s.X=(aVertices[iVertex][0] + 2048) << 4;
        pRegister->XYZ2.s.Y=(aVertices[iVertex][1] + 2048) << 4;
        //pRegister->XYZ2.s.X=(2048 << 4) + ((aVertices[iVertex][0]*(1 & 31)) >> 1);
        //pRegister->XYZ2.s.Y=(2048 << 4) + ((aVertices[iVertex][1]*(1 & 31)) >> 1);
        pRegister->XYZ2.s.Z=z;
        
        // Increment register data pointer
        pRegister++;
    }

    // Compute the difference between our register data pointer and our initial
    // pointer.  This is the byte count of data we put in.  Shifted right four
    // times gives us the qword count
    return ((int) pRegister -(int) pvBase) >> 4;
}

