//InputSystem.cpp
//
//Implementation of InputSystem class

#include "InputSystem.h"

InputSystem::InputSystem(PoolRoom *ptrPoolRoom, Camera *ptrCamera, SoundSystem *ptrSoundSystem, GameRules *ptrGameRules, HUD *ptrHUD, GameManager *ptrGameManager)
{
    joypadlib_initialize();

    smallForce    = 55.0f;
	largeForce    = 95.0f;
	megaForce     = 120.0f;
    zoomSpeed     = 1000.0;
    
    menuPressedToggle   = false;
    musicPressedToggle  = false;
    resetPressedToggle  = false;
    menuUpToggle        = false;
    menuDownToggle      = false;
    menuSelectToggle    = false;
    shootBtnPressed     = false;
    aiplayer            = false;
    menuToggleInterval  = 5.0;
    elapsedMenuInterval = 0.0;
    shootingForce       = 0.0;
    shootingForceInc    = 1.0;
    prevGamestate       = GAMESTATE_INTRO;

    megaButton       = PS2PAD_BUTTON_TRIANGLE;
    largeButton      = PS2PAD_BUTTON_CIRCLE;
    smallButton      = PS2PAD_BUTTON_CROSS;
    menuButton       = PS2PAD_BUTTON_START;
	musicButton      = PS2PAD_BUTTON_SQUARE;
    menuSelectButton = PS2PAD_BUTTON_CROSS;
    menuDownButton   = PS2PAD_BUTTON_DOWN;
    menuUpButton     = PS2PAD_BUTTON_UP;
    aimStick         = &leftStick[0];
    zoomStick        = &rightStick[0];

    poolRoom      = ptrPoolRoom;
    whiteBall     = poolRoom->GetWhiteBall();
    camera        = ptrCamera;
    soundSystem   = ptrSoundSystem;
    gameRules     = ptrGameRules;
	hud			  = ptrHUD;
    gameManager   = ptrGameManager;

    hud->setMaxForce(megaForce);

	bob = new AI(gameRules, gameManager, poolRoom);
}

InputSystem::~InputSystem()
{
	delete bob;
    joypadlib_shutdown();    		// shut down joypadlib
	printf("joypadlib shutdown\n");
}

float InputSystem::getShootingForce()
{
    return shootingForce;
}

void InputSystem::CheckInput(int &gamestate) //Pass in current game state, return new game state
{
    //Get joypad input
    joypadlib_update();
	buttons = joypadlib_get_joypad_buttons(0, NULL, PRESSURE_TYPE_NONE);
	joypadlib_get_joypad_sticks(0, leftStick, rightStick);

	//Handle menu/music button presses here
    if(musicPressedToggle && !(buttons & musicButton))
        musicPressedToggle = false;
    if(menuPressedToggle && !(buttons & menuButton))
        menuPressedToggle  = false;

	//Handle menu up/down button presses here
    if(menuUpToggle && !(buttons & menuUpButton))
        menuUpToggle       = false;
    if(menuDownToggle && !(buttons & menuDownButton))
        menuDownToggle     = false;
    if(menuSelectToggle && !(buttons & menuSelectButton))
        menuSelectToggle   = false;

    if(menuUpToggle)
    {
        if((buttons & menuUpButton) && (elapsedMenuInterval > menuToggleInterval))
        {
            menuUpToggle        = false;
            elapsedMenuInterval = 0.0;
        }
        else
            elapsedMenuInterval += 1.0;
    }
    if(menuDownToggle)
    {
        if((buttons & menuDownButton) && (elapsedMenuInterval > menuToggleInterval))
        {
            menuDownToggle        = false;
            elapsedMenuInterval = 0.0;
        }
        else
            elapsedMenuInterval += 1.0;
    }

    //if(buttons & PS2PAD_BUTTON_R2) gamestate = GAMESTATE_EXITING; //emergency exit.

    switch(gamestate)
    {
        case GAMESTATE_INTRO:
        {
            if((buttons & menuButton) && !menuPressedToggle)
            {
                gamestate         = GAMESTATE_MENU;
                prevGamestate     = GAMESTATE_INTRO;
                menuPressedToggle = true;
            }
            break;
        }

        case GAMESTATE_OUTRO:
        {
            if((buttons & menuButton) && !menuPressedToggle)
            {
                gamestate         = GAMESTATE_EXITING;
                prevGamestate     = GAMESTATE_OUTRO;
                menuPressedToggle = true;
            }
            break;
        }

        case GAMESTATE_WAITING:
        {
            // AI player
            if(aiplayer && gameManager->GetCurrentPlayer()->GetPlayerNumber() == 2)
            {
                camera->RestrictCamera(false);
            	bob->Play(whiteBall);
				gamestate = GAMESTATE_PLAYING;
            }

            camera->GetCameraVector(shotVec); //Only need to do this if the player can actually use the cue

            if((buttons & smallButton) && !menuSelectToggle)
			{
                if (!shootBtnPressed)
                {
                    shootBtnPressed = true;
                    shootingForce = 0.0;
                    if(shootingForceInc < 0.0)
                        shootingForceInc = -shootingForceInc;
                }
                shootingForce += shootingForceInc;
                if((shootingForce >= megaForce) || (shootingForce <= 0.0))
                    shootingForceInc = -shootingForceInc;
                hud->setShootingForce(shootingForce);
            }

            if (!((buttons & smallButton) && !menuSelectToggle) && shootBtnPressed)
            {
                shootBtnPressed = false;
                whiteBall->AddForce(shootingForce * shotVec[0], shootingForce * shotVec[1], shootingForce * shotVec[2]);
                poolRoom->GetUtensil()->SetMaxCueDistance(shootingForce / megaForce * 1500.0);
                poolRoom->GetUtensil()->ResetAnimation();
				gamestate = GAMESTATE_ANIMATECUE;
                shootingForce = -1.0;
                hud->setShootingForce(shootingForce);
                camera->RestrictCamera(false);
            }

            if (aimStick[0] < 0.0)
                camera->RotateCamera(0.0, PI/180 * aimStick[0] * -aimStick[0], 0.0);
            if (aimStick[0] > 0.0)
                camera->RotateCamera(0.0, PI/180 * aimStick[0] * aimStick[0], 0.0);
			    
			if ((zoomStick[1] < 0.0) || (zoomStick[1] > 0.0))
				camera->Zoom(-zoomStick[1] * zoomSpeed);

            if((buttons & menuButton) && !menuPressedToggle)
            {
                gamestate         = GAMESTATE_MENU;
                prevGamestate     = GAMESTATE_WAITING;
                menuPressedToggle = true;
            }

            if((buttons & musicButton) && !musicPressedToggle)
            {
                camera->RestrictCamera(false);
				bob->Play(whiteBall);
				gamestate = GAMESTATE_PLAYING;
            }

            break;
        }            

        case GAMESTATE_PLAYING:
        {
            if((buttons & menuButton) && !menuPressedToggle)
            {
                gamestate         = GAMESTATE_MENU;
                prevGamestate     = GAMESTATE_PLAYING;
                menuPressedToggle = true;
            }
            
            if (aimStick[0] < 0.0)
				camera->RotateCamera(0.0, PI/180 * aimStick[0] * 5.0f, 0.0);
			if (aimStick[0] > 0.0)
				camera->RotateCamera(0.0, PI/180 * aimStick[0] * 5.0f, 0.0);
			    
			if ((zoomStick[1] < 0.0) || (zoomStick[1] > 0.0))
				camera->Zoom(-zoomStick[1] * zoomSpeed);

            break;
        }

        case GAMESTATE_MENU:
        {
            if((buttons & menuButton) && !menuPressedToggle && (prevGamestate != GAMESTATE_INTRO))
            {
            	menuPressedToggle = true;
                gamestate         = prevGamestate;
            }
			
            if((buttons & menuSelectButton) && !menuSelectToggle)
			{
                menuSelectToggle = true;
				switch(hud->getCurrentSelection())
				{
                    case MENU_START1PLAYER:
                    {
                        aiplayer = true;
                        gameManager->StartNewGame();
                        gamestate = GAMESTATE_WAITING;
                        break;
                    }
					case MENU_START2PLAYERS:
                    {
                        aiplayer = false;
                        gameManager->StartNewGame();
                        gamestate = GAMESTATE_WAITING;
                        break;
                    }
                    case MENU_EXIT:
                    {
                        gamestate = GAMESTATE_OUTRO;
                        break;
                    }
				}
			}
			
            if((buttons & menuDownButton) && !menuDownToggle)
            {
                hud->changeMenuItem(1);
                menuDownToggle      = true;
                elapsedMenuInterval = 0.0;
            }

			if((buttons & menuUpButton) && !menuUpToggle)
            {
                hud->changeMenuItem(-1);
                menuUpToggle        = true;
                elapsedMenuInterval = 0.0;
            }

            break;
        }

        case GAMESTATE_EXITING:
            break;

        case GAMESTATE_SHOWRESULTS:
        {
            if((buttons & menuButton) && !menuPressedToggle)
            {
            	menuPressedToggle = true;
                prevGamestate = GAMESTATE_SHOWRESULTS;
                gamestate = GAMESTATE_MENU;
            }
        }

        case GAMESTATE_ANIMATECUE:
            break;

        default:
        {
            printf("Uh oh... Invalid game state [%i]\n",gamestate);
            break;
        }
    }
}
