//Model.cpp

#include "Model.h"

Model::Model(RenderManager* ptrRMgr)
{
	hasTextures    = false;
    modelHasLoaded = false;
    renderMgr      = ptrRMgr;
}

Model::~Model()
{}

void Model::LoadModel(char* modelName)
{
	char scene[50] = "./models/scene_";
	char geo[50] = "./models/geomdata_";
	strcat(scene, modelName);
	strcat(scene, ".asc");
	strcat(geo, modelName);
	strcat(geo, ".bin");
	CSceneCreatorIntmd scenecreatorobject = Render_LoadScene(scene, geo);
	numinstances = LoadObjects(scenecreatorobject, meshaddr, textureaddr, scenetransform);
	scenecreatorobject.CleanUp(); // free memory

    modelHasLoaded = true;
}

int Model::LoadObjects(CSceneCreatorIntmd scenecreatorobject, void *meshpointers[], void *texturepointers[], Mat44 transforms[])
{
   CSceneIntmd sceneobject = scenecreatorobject.GetSceneIntmd();
   int numinstancess = sceneobject.GetNumMeshInstances();
   for (int instancenum = 0; instancenum < numinstancess; instancenum++)
	{
      // load meshes
      CMeshInstanceIntmd meshinstanceobject = sceneobject.GetMeshInstance(instancenum);
      Vec3 translation = meshinstanceobject.GetPosKey(0);
      transinfo transform;
      transform.translationx = translation.x;
      transform.translationy = translation.y;
      transform.translationz = translation.z;
      transforms[instancenum] = Render_MakeObjectToWorld(transform);
      meshpointers[instancenum] = Render_LoadMesh(0, scenecreatorobject, meshinstanceobject.GetMeshIndex());
      // load textures
      CMeshIntmd meshobject = sceneobject.GetMesh(meshinstanceobject.GetMeshIndex());
      CSurfaceIntmd surfaceobject = meshobject.GetSurface(0);
      CMaterialIntmd materialobject = sceneobject.GetMaterial(surfaceobject.GetMaterialIndex());
      CTextureSeqIntmd textureseqobject = materialobject.GetDiffTexture();
	
		if (textureseqobject.GetNumTexFrames() != 0)
		{
			hasTextures = true;
			texturepointers[instancenum] = Render_LoadTexture(scenecreatorobject,textureseqobject.GetTexFrameIndex(0));
		}
   }
   return(numinstancess);
}

void Model::DrawModel(Mat44 &rotation, Mat44 &worldtoscreen, lightinfo &lights)
{
   if( modelHasLoaded )
   {
      // create dma chain in currbuffer
      for (int instancenum = 0; instancenum < numinstances; instancenum++)
	   {
         objecttoworld = rotation * scenetransform[instancenum];

		   Render_AddMatrix(worldtoscreen * objecttoworld, 0);
		   Render_AddVector(SCALE_VECTOR,4);
		   Render_AddMatrix(Render_MakeLightMatrix(lights, objecttoworld), 5);
   		
         if (hasTextures)
		      Render_AddCallTag(textureaddr[instancenum]);
	      Render_AddCallTag(meshaddr[instancenum]);
      }

       renderMgr->DrawingObject();
   }
}

void Model::DrawModel(Mat44 &rotation, Mat44 &worldtoscreen, lightinfo &lights, bool kickFrame)
{
    DrawModel(rotation, worldtoscreen, lights);
    if(kickFrame)
        renderMgr->FinishedFrame();
}
