#include "PlayingBall.h"
#include <math.h>

PlayingBall::PlayingBall(dWorldID world, dSpaceID space, ModelController *modelController, GeomController *geomController, float ballSize, float ballMass, char* modelName, int ballNo, char* ballName, int type) : CObject(modelName, modelController, ballName)
{
	dMassSetSphere(&mass, 1, ballSize);
	dMassAdjust(&mass, ballMass);

	ballGeom = dCreateSphere(space, ballSize);
	bodyID   = dBodyCreate(world);
	dBodySetMass(bodyID, &mass);
	dGeomSetBody(ballGeom, bodyID);
	dBodySetPosition(bodyID, 0.0, 0.0, 0.0);
	geomController->AddGeom(ballGeom, "Ball", this);

    ballNumber        = ballNo;
    isStopped         = true;
    ballType          = type;
    score             = 0;
    respawn           = false;
    respawnFromPocket = false;

    dQuaternion q;
    dQSetIdentity(q);
    dBodySetQuaternion(bodyID, q);

}

PlayingBall::~PlayingBall()
{}

bool PlayingBall::IsStopped()
{
    return isStopped;
}

void PlayingBall::UpdateVelocityStatus()
{
    if( bodyID != NULL )
    {
        const dReal *currentVel = dBodyGetLinearVel(bodyID);
	    float linVel = sqrt((currentVel[0] * currentVel[0]) + (currentVel[1] * currentVel[1]) + (currentVel[2] * currentVel[2]));
        
        if(linVel <= CUTOFF_VELOCITY)
        {
            isStopped = true;
            StopMovement();
        }
        else
            isStopped = false;
    }
}

int PlayingBall::GetBallType()
{
    return ballType;
}

void PlayingBall::SetRespawnStatus(bool rStatus)
{
    respawn = rStatus;
}

bool PlayingBall::GetRespawnFromPocketStatus()
{
    return respawnFromPocket;
}

void PlayingBall::SetRespawnFromPocketStatus(bool rStatus)
{
    respawnFromPocket = rStatus;
}

void PlayingBall::Respawn()
{
    if(respawn)
    {
        SetPhysicalPosition(respawnCoords[0], respawnCoords[1], respawnCoords[2]);
        StopMovement();
        respawn  = false;
        objState = ObjOnTable;
    }
}

void PlayingBall::RemoveFromTable()
{
    SetPhysicalPosition(removalCoords[0], removalCoords[1], removalCoords[2]);
    StopMovement();
}

void PlayingBall::SetRespawnCoords(float x, float y, float z)
{
    respawnCoords[0] = x;
    respawnCoords[1] = y;
    respawnCoords[2] = z;
}

void PlayingBall::SetRemovalCoords(float x, float y, float z)
{
    removalCoords[0] = x;
    removalCoords[1] = y;
    removalCoords[2] = z;
}
