#include "PoolRoom.h"

PoolRoom::PoolRoom(GameRules *rules, dWorldID world, dSpaceID space, ModelController *modelController, GeomController *geomController) : CObject(NULL, NULL, "PoolRoom")
{
    int i;
    
	gameRules = rules;

	//Create the playing surface
    playingSurface = new PlayingSurface(space, modelController, geomController, gameRules->GetSurfaceModel());
	this->Attach(playingSurface);

    //Create the utensil
    utensil = new Utensil(space, modelController, geomController, gameRules->GetUtensilModel());

    //Do the load according to GameRules stuff here
    for( i = 0; i < gameRules->GetNumberBalls(); ++i )
    {
		playingBalls[i] = new PlayingBall(world, space, modelController, geomController, gameRules->GetBallSize(i), gameRules->GetBallMass(i), gameRules->GetBallModel(i), i, gameRules->GetBallName(i), gameRules->GetBallType(i));
        if(playingBalls[i]->GetBallType() == BALL_WHITE)
            playingBalls[i]->SetRespawnFromPocketStatus(true);

        playingBalls[i]->SetRespawnCoords(gameRules->GetBallX(i), gameRules->GetBallY(i), gameRules->GetBallZ(i));
        playingBalls[i]->SetRemovalCoords(gameRules->GetBallX(i), gameRules->GetBallY(i) + gameRules->GetRemovalOffset(), gameRules->GetBallZ(i));

        if (i == 0)
			this->Attach(playingBalls[i]);
		else
			playingBalls[i-1]->Attach(playingBalls[i]);
    }

    for( i = 0; i < gameRules->GetNumberPockets(); ++i )
    {
        pockets[i] = new Pocket(gameRules->GetPocketX(i), gameRules->GetPocketY(i), gameRules->GetPocketZ(i), gameRules->GetPocketRadius(i), space, geomController, gameRules->GetPocketName(i));
        playingSurface->Attach(pockets[i]);
    }

    ResetBallPositions();

	//AI Stuff
	target = new CObject("ballcue", modelController, "Target");
    target->SetModelPosition(0,-1000,0);
    targetPocket = new CObject("ballcue", modelController, "TargetPocket");
    targetPocket->SetModelPosition(0,-1000,0);
    targetBall = new CObject("ballcue", modelController, "TargetBall");
    targetBall->SetModelPosition(0,-1000,0);
    startingPoint = new CObject("ballcue", modelController, "StartingPoint");
    startingPoint->SetModelPosition(0,-1000,0);
}

PoolRoom::~PoolRoom()
{
    delete utensil; //Not attached to main room tree structure - MUST be deleted separately
    delete target;
    delete targetPocket;
    delete targetBall;
    delete startingPoint;
}

void PoolRoom::DrawUtensil(Mat44 worldtoscreen, lightinfo lights)
{
    utensil->DrawObject(worldtoscreen, lights);
}

void PoolRoom::ResetBallPositions()
{
	for( int i = 0; i < gameRules->GetNumberBalls(); ++i )
	{
		playingBalls[i]->SetPhysicalPosition(gameRules->GetBallX(i), gameRules->GetBallY(i), gameRules->GetBallZ(i));
		playingBalls[i]->StopMovement();
	}
}

PlayingBall* PoolRoom::GetWhiteBall()
{
    return playingBalls[0];
}

void PoolRoom::UpdateBallVelocities()
{
    for( int i = 0; i < gameRules->GetNumberBalls(); ++i )
        playingBalls[i]->UpdateVelocityStatus();    
}

bool PoolRoom::AllBallsStopped()
{
    bool areBallsStopped = true;

    for( int i = 0; i < gameRules->GetNumberBalls(); ++i )
	{
        if(!(playingBalls[i]->IsStopped()))
            areBallsStopped = false;
	}

    return areBallsStopped;
}

PlayingSurface* PoolRoom::GetPlayingSurface()
{
    return playingSurface;
}

void PoolRoom::RespawnBalls()
{
    for(int i = 0; i < gameRules->GetNumberBalls(); ++i)
        playingBalls[i]->Respawn();
}

PlayingBall* PoolRoom::GetPlayingBall(int ballNo)
{
    return playingBalls[ballNo];
}

void PoolRoom::ResetAllBalls()
{
    for(int i = 0; i < gameRules->GetNumberBalls(); ++i)
        playingBalls[i]->SetRespawnStatus(true);
    RespawnBalls();
}

Utensil* PoolRoom::GetUtensil()
{
    return utensil;
}
