#include "TextureManager.h"

TextureManager::TextureManager(int sps2d)
{
	for (int i = 0; i < MAX_BITMAPS; i++)
	{
		bitmap[i].w=0;
        bitmap[i].h=0;
        bitmap[i].loaded=false;
	}
    desc=sps2d;
}

TextureManager::~TextureManager()
{
    for(int i = 0; i < MAX_BITMAPS; ++i)
        if((bitmap[i].loaded) && (bitmap[i].pMemCached != NULL))
            sps2Free(bitmap[i].pMemCached);
}

bool TextureManager::LoadFile(char *fileName, bool trans)
{
    char tmpFileName[100];
    sprintf(tmpFileName, "./images/%s", fileName);
	//create new instance of image.
    if (desc != -1)
    {
        for (int i = 0; i < MAX_BITMAPS; i++)
        {
            if (!bitmap[i].loaded)
            {
                if(!LoadBitmap(tmpFileName, &(bitmap[i]), trans, desc))
                {
		            printf("[FAILED]\n");
                    exit(0);
                }

				bitmap[i].name=fileName;
                bitmap[i].loaded=true;

                return true;
            }
        }
        return false;
    }
    else
    {
        printf("desc was -1\n");
        exit(0);
    }
}

void TextureManager::DrawBitmap(char *fileName, int x, int y, int z)
{
	//GET CORRECT STRUCT
    for (int i = 0; i < MAX_BITMAPS; i++)
    {
        if (bitmap[i].loaded)
        {
            if (strcmp(bitmap[i].name,fileName)==0)
            {
                Upload(sps2UScreenGetFirstFreeGSPage(), &(bitmap[i].pcMemory[TEXTUREMEM_256K]), &(bitmap[i].pcMemory[bitmap[i].texDmaMemNeeded]),bitmap[i].pMemCached,desc,bitmap[i].w,bitmap[i].h);
    	        bitmap[i].iQWC = BuildGSPacket((void *)&(bitmap[i].pcMemory[PRIMITIVEMEM_4K]), x, y, z, bitmap[i].w, bitmap[i].h);
	            bitmap[i].madr.i32 = 0;		
	            bitmap[i].madr.s.ADDR = sps2GetPhysicalAddress((void *)&(bitmap[i].pcMemory[PRIMITIVEMEM_4K]), bitmap[i].pMemCached);
	            bitmap[i].chcr.i32 = 0;
	            bitmap[i].chcr.s.MOD = CHCR_MOD_NORMAL;	// Normal mode transfer
	            bitmap[i].chcr.s.STR = 1;					// Start the transfer
	            *EE_D2_QWC = bitmap[i].iQWC;
	            *EE_D2_MADR = bitmap[i].madr.i32;
	            *EE_D2_CHCR = bitmap[i].chcr.i32;
	            sps2WaitForDMA(2, desc);

                i = MAX_BITMAPS;
            }
        }
    }
}


//thanks mr. fortuna!
int TextureManager::BuildGSPacket(void * pMem, int x, int y, int z, int w, int h)
{
	sps2GIFTag_t * pGIFTag = (sps2GIFTag_t *)pMem;// Declare a GIFTag pointer
	// Create a textured sprite
	pGIFTag->i128 = 0;
	pGIFTag->s.NLOOP = 1;	// Only one of them
	pGIFTag->s.PRE = 1;		// Transfer PRIM to the GS
	pGIFTag->s.EOP = 1;		// This is the last giftag in the packet
	// Set up the PRIM register with texture mapping on and UV coordinates
	pGIFTag->s.PRIM = GS_PRIM(PRIM_SPRITE,PRIM_IIP_FLAT,PRIM_TME_ON,PRIM_FGE_OFF, 
							PRIM_ABE_ON,PRIM_AA1_OFF,PRIM_FST_UV,PRIM_CTXT_CONTEXT1, 
							PRIM_FIX_NOFIXDDA);
	pGIFTag->s.FLG = GIF_FLG_PACKED;
	// This is 5 now, as we are sending one colout two positions and two UV coordinates
	pGIFTag->s.NREG = 5; 
	pGIFTag->s.REGS = GIF_REG_RGBAQ | (GIF_REG_UV << 4) | (GIF_REG_XYZ2 << 8) | (GIF_REG_UV << 12) 	| (GIF_REG_XYZ2 << 16);
	// Set the register pointer to the memory just after the GIFTag
	// pGIFTag + 1 will move the address of the pointer
	// ahead by the sizeof(sps2GIFTag_t) which is 1 quadword (128 bits)
	sps2GIFPackedRegister_t	* pRegister = (sps2GIFPackedRegister_t *)(pGIFTag + 1);
	// Add the register data to the GS Packet
	AddColour(pRegister, 0x80, 0x80, 0x80, 0x80);// RGBA
	AddUV(pRegister, 0, 0);// UV
	AddVertex(pRegister, -320 + x, -240 + y, z);// XYZ
	AddUV(pRegister, w, h);// UV
	AddVertex(pRegister, (-320 + x) +  w,  (-240 + y) + h, z);// XYZ
	// Work out how many quad words we just wrote
	int iQWC = ((int)pRegister - (int)pMem) >> 4;
	// Assert that we haven't written more data than we allocated
	assert(iQWC < (4096 >> 4));
	// Important: Make sure all the data is written to the physical memory
	sps2FlushCache(desc);	
	return iQWC;	//return the number of qwords written
}

void TextureManager::DeleteBitmap(char* fileName)
{
    for(int i = 0; i < MAX_BITMAPS; ++i)
    {
        if(strcmp(bitmap[i].name,fileName) == 0)
        {
            if((bitmap[i].loaded) && (bitmap[i].pMemCached != NULL))
                sps2Free(bitmap[i].pMemCached);
		    bitmap[i].w=0;
            bitmap[i].h=0;
            bitmap[i].loaded=false;
        }
    }
}
