#include "Utensil.h"

Utensil::Utensil(dSpaceID space, ModelController *modelController, GeomController *geomController, char *modelName) : CObject(modelName, modelController, "Utensil")
{
    currentCueDistance = 0.0;
    maxCueDistance     = 1.0;
    animationLength    = 1;
    strikeLength       = 1;
    animationIsRunning = false;
    fwdCueStep         = maxCueDistance;
    backCueStep        = maxCueDistance;
    cueDir             = 1; //Away from ball
}

Utensil::~Utensil()
{}

void Utensil::DrawUtensil(Mat44 worldtoscreen, lightinfo lights, float rotationAngle, float centreX, float centreY, float centreZ)
{
    LoadIdentity(&mat1);
    LoadIdentity(&mat2);
    LoadIdentity(&mat3);

    LoadRotation(&mat1, CUEANGLE, 0.0, 0.0);
    SetColumn(&mat1, 3, Vec4(0.0, 0.0, currentCueDistance, 1.0));
    SetColumn(&mat2, 3, Vec4(0.0, 5500.0, 4000.0, 1.0));
    LoadRotation(&mat3, 0.0, -rotationAngle, 0.0);
    SetColumn(&mat3, 3, Vec4(centreX, centreY, centreZ, 1.0));

    rotation = mat3 * mat2 * mat1;
    model->DrawModel(rotation, worldtoscreen, lights);
}

void Utensil::SetMaxCueDistance(float newMaxDistance)
{
    maxCueDistance = newMaxDistance;
    if(cueDir == 1)
    {
        fwdCueStep     = (maxCueDistance - currentCueDistance) / animationLength;
        backCueStep    = maxCueDistance / strikeLength;
    }
    else
    {
        fwdCueStep     = maxCueDistance / animationLength;
        backCueStep    = (maxCueDistance - currentCueDistance) / strikeLength;
    }
}

void Utensil::SetAnimationLength(int lenInFrames)
{
    animationLength = lenInFrames;
    if(cueDir == 1)
        fwdCueStep = (maxCueDistance - currentCueDistance) / animationLength;
    else
        fwdCueStep = maxCueDistance / animationLength;
}

void Utensil::ResetAnimation()
{
    currentCueDistance = 0.0;
    cueDir             = 1;
}

bool Utensil::AnimationIsComplete()
{
    return !animationIsRunning;
}

void Utensil::DoAnimationStep()
{
    animationIsRunning = true;
    
    if(cueDir == 1)
    {
        if(currentCueDistance > maxCueDistance)
            cueDir = 2; //Towards ball
        else
            currentCueDistance += fwdCueStep;
    }
    else
        if(cueDir == 2)
        {
            if(currentCueDistance <= 0.0)
                animationIsRunning = false;
            else
                currentCueDistance -= backCueStep;
        }
}

void Utensil::SetStrikeLength(int lenInFrames)
{
    strikeLength = lenInFrames;
    if(cueDir == 1)
        backCueStep = maxCueDistance / strikeLength;
    else
        backCueStep = (maxCueDistance - currentCueDistance) / strikeLength;
}
