#include "sock.h"

SocketController::SocketController()
{
	gethostname(domain, sizeof(domain));
	if ((servSock = socket(AF_INET, SOCK_STREAM, 0)) < 0) printf("socket() failed");
    memset(&ServAddr, 0, sizeof(ServAddr));   		/* Zero out structure */
    ServAddr.sin_family = AF_INET;                	/* Internet address family */
    ServAddr.sin_addr.s_addr = htonl(INADDR_ANY); 	/* Any incoming interface */
    ServAddr.sin_port = htons(PORT);      		/* Local port */
	if (bind(servSock, (struct sockaddr *) &ServAddr, sizeof(ServAddr)) < 0)
	{
		close(servSock);
		servSock = -1;
	}
	else
	{
		if (listen(servSock, 5) < 0)
		{
			close(servSock);
			servSock = -1;
		}
		printf("[created] sock:%d(%s)\n",servSock,domain);
		clntSock = 0;
		flag = 1;
		reuse_addr = 1;
		setsockopt(servSock, SOL_SOCKET, SO_REUSEADDR, &reuse_addr,sizeof(reuse_addr));
		SetNonBlocking(servSock);
		FD_ZERO(&readfds);
		FD_ZERO(&master);
		FD_SET(servSock, &readfds);
		max = servSock;
	}
}

SocketController::~SocketController()
{
	close(servSock);
	if (clntSock > 0) close(clntSock);
}

void SocketController::CloseConnection()
{
	if (clntSock > 0)
	{
		close(clntSock);
		FD_CLR(clntSock, &readfds);
		max = servSock;
	}
}

int SocketController::SendMessage(char msg[BUFLEN])
{
	if (clntSock > 0)
		return send(clntSock, msg, strlen(msg), 0);
	else
		return 1;
}

int SocketController::CheckMessages(char msg[BUFLEN])
{
	master = readfds;
	tv.tv_sec=0;
	tv.tv_usec=0;
	int ret = select(max+1, &master, NULL, NULL, &tv);

	if (ret<0)
	{
		printf("Select busted\n");
		ret = 0;
	}	
	else if (ret > 0)
	{
		if (FD_ISSET(servSock, &master))
		{
			clntLen = sizeof(ClntAddr);
			if ((clntSock = accept(servSock, (struct sockaddr *) &ClntAddr, &clntLen)) < 0)
				printf("accept() failed");
			printf("Handling client %s[%d]\n", inet_ntoa(ClntAddr.sin_addr), clntSock);
			SetNonBlocking(clntSock);
			FD_SET(clntSock, &readfds);
			max = clntSock;
			ret = 3;
			printf("Handling client %s[%d]\n", inet_ntoa(ClntAddr.sin_addr), clntSock);
			sprintf(msg, "msg|Welcome to the PlayStation2 on %s~",domain); 
			SendMessage(msg);
			msg[0]='\0';
		}
		else if (FD_ISSET(clntSock, &master))
		{
    			int recvMsgSize;
			if ((recvMsgSize=recv(clntSock, msg, BUFLEN, 0)) <= 0)
			{
				if (recvMsgSize==0)
				{
					FD_CLR(clntSock,&readfds);	
					close(clntSock);
					printf("DEAD!");
					ret = 0;
					max = servSock;
				}
			} else {
				msg[recvMsgSize]='\0';
				printf("--%s==\n",msg);
				ret = 1;
			}
		}
		else ret = 0;
	}
	else 
		ret = -1;

	fflush(stdout);
	return ret;
}

void SocketController::SetNonBlocking(int sock)
{
	int opts;
	opts = fcntl(sock,F_GETFL);
	if (opts < 0) printf("fcntl(F_GETFL)");
	opts = (opts | O_NONBLOCK);
	if (fcntl(sock,F_SETFL,opts) < 0) printf("fcntl(F_SETFL)");
	return;
}
